#include"MPU.h"
#include"I2C.h"
#include"p24Exxxx.h"

#define FCY 70000000LL 
#include <libpic30.h>




void MPUInit()
{
	__delay_ms(150);	//Wait for wake up
	
	// Initialize MPU6050 device
	// wake up device
	MPUWrite(MPU6050_ADDRESS, PWR_MGMT_1, 0x00); // Clear sleep mode bit (6), enable all sensors 
	__delay_ms(100); // Delay 100 ms for PLL to get established on x-axis gyro; should check for PLL ready interrupt  
	
	// get stable time source
	MPUWrite(MPU6050_ADDRESS, PWR_MGMT_1, 0x01);  // Set clock source to be PLL with x-axis gyroscope reference, bits 2:0 = 001
	
	// Configure Gyro and Accelerometer
	// Disable FSYNC and set accelerometer and gyro bandwidth to 44 and 42 Hz, respectively; 
	// DLPF_CFG = bits 2:0 = 010; this sets the sample rate at 1 kHz for both
	// Maximum delay is 4.9 ms which is just over a 200 Hz maximum rate
	MPUWrite(MPU6050_ADDRESS, CONFIG, 0x03);  
	
	// Set sample rate = gyroscope output rate/(1 + SMPLRT_DIV)
	MPUWrite(MPU6050_ADDRESS, SMPLRT_DIV, 0x04);  // Use a 200 Hz rate
	
	// Set gyroscope full scale range
	MPUWrite(MPU6050_ADDRESS, GYRO_CONFIG, 0 ); // Set full scale 250dps
	
	// Set accelerometer configuration
	MPUWrite(MPU6050_ADDRESS, ACCEL_CONFIG, 0); // Set full scale 2g 
	
	
	// Configure Interrupts and Bypass Enable
	// Set interrupt pin active high, push-pull, and clear on read of INT_STATUS, enable I2C_BYPASS_EN so additional chips 
	// can join the I2C bus and all can be controlled by the Arduino as master
	MPUWrite(MPU6050_ADDRESS, INT_PIN_CFG, 0x10);    
	MPUWrite(MPU6050_ADDRESS, INT_ENABLE, 0x01);  // Enable data ready (bit 0) interrupt

}

void MPUWrite(unsigned char deviceAddress, unsigned char address, unsigned char val)
{
    I2CStart(deviceAddress); 
	I2CSend(address);   
	I2CSend(val);   
	I2CStop();   
}

unsigned char MPURead(unsigned char deviceAddress, unsigned char address)
{
    I2CStart(deviceAddress); 	
	I2CSend(address);
	I2CReStart(deviceAddress | 0b00000001);	//When read gyro sens, set BIT0 !
	I2C1CONbits.RCEN = 1;
	I2CWait();
	I2C1CONbits.ACKDT = 1;	//Send NACK
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.ACKDT = 0;
	I2CStop();
    return I2C1RCV;
}

void MPUReadGyroData(unsigned char deviceAddress, short* Data)
{
	unsigned char temp[6];
    I2CStart(deviceAddress); 	
	I2CSend(GYRO_XOUT_H);
	I2CReStart(deviceAddress | 0b00000001);	//When read gyro sens, set BIT0 !
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[0]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[1]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[2]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[3]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[4]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[5]= I2C1RCV;
	I2C1CONbits.ACKDT = 1;	//Send NACK
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.ACKDT = 0;
	I2CStop();

	*Data = temp[0];
	*Data = *Data << 8;
	*Data = *Data | temp[1];
	Data++;
	*Data = temp[2];
	*Data = *Data << 8;
	*Data = *Data | temp[3];
	Data++;
	*Data = temp[4];
	*Data = *Data << 8;
	*Data = *Data | temp[5];
    
}

void MPUReadAccelData(unsigned char deviceAddress, short* Data)
{
	unsigned char temp[6];
    I2CStart(deviceAddress); 	
	I2CSend(ACCEL_XOUT_H);
	I2CReStart(deviceAddress | 0b00000001);	//When read gyro sens, set BIT0 !
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[0]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[1]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[2]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[3]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[4]= I2C1RCV;
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.RCEN = 1;
	I2CWait();
	temp[5]= I2C1RCV;
	I2C1CONbits.ACKDT = 1;	//Send NACK
	I2C1CONbits.ACKEN = 1;
	I2CWait();
	I2C1CONbits.ACKDT = 0;
	I2CStop(); 

	*Data = temp[0];
	*Data = *Data << 8;
	*Data = *Data | temp[1];
	Data++;
	*Data = temp[2];
	*Data = *Data << 8;
	*Data = *Data | temp[3];
	Data++;
	*Data = temp[4];
	*Data = *Data << 8;
	*Data = *Data | temp[5];  
}


