#include"p24Exxxx.h"
#include <stdlib.h> 
#include <string.h>
#include"I2C.h"
#include"MPU.h"
#include "LCD.h"
#include "bluetooth.h"
#include "complementary_filter.h"


#define BLUETOOTH_TX LATBbits.LATB4	//To Bluetooth RX 
#define STATUS_LED 	LATBbits.LATB5	//Blink when MPU interrupt
#define LCD_CLOCK LATBbits.LATB11	//LCD SPI PINs
#define LCD_DATA LATBbits.LATB10

#define FCY 64000000LL            //Define clock
#include <libpic30.h>            //rajel fggvnye, a define FCY alatt kell lennie, klnben nem mukdik

_FPOR(ALTI2C1_ON)	//I2C alternativ pin configuration  :RB8, RB9  
_FGS(GWRP_OFF & GCP_OFF)
_FICD(ICS_PGD1 & JTAGEN_OFF)
_FWDT(FWDTEN_OFF)				//Watchdog off
_FOSCSEL(FNOSC_PRIPLL);			// Primary XT Oscillator 4Mhz chrystal
_FOSC(FCKSM_CSDCMD & OSCIOFNC_OFF  & POSCMD_XT);  
								// Clock Switching and Fail Safe Clock Monitor is disabled
								// OSC2 Pin Function: OSC2 is Clock Output
								// Primary Oscillator Mode: XT Crystal


void writeRegister(int deviceAddress, unsigned char address, unsigned char val);	//Send data to MPU
int readRegister(int deviceAddress, unsigned char address);							//Read data from MPU



 void init()
{
	CLKDIV = 0;
	// Configure PLL prescaler, PLL postscaler, PLL divisor	
	CLKDIVbits.PLLPOST=0b00; // N2=2
	CLKDIVbits.PLLPRE=0b00; // N1=2
	CLKDIVbits.DOZEN = 0;
	PLLFBD=126; // M=128

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC=0b011)
	__builtin_write_OSCCONH(0x03);
	__builtin_write_OSCCONL(OSCCON | 0x01);
	// Wait for Clock switch to occur
	while (OSCCONbits.COSC!= 0b011);
	// Wait for PLL to lock
	while (OSCCONbits.LOCK!= 1);

   	ANSELA = 0;                //Disable analog pin
	ANSELB = 0;
    TRISB = 0;					//All port output
    TRISA = 0;

	TRISBbits.TRISB7 = 1;	//Interrupt PIN for data avaible on MPU
	IFS0bits.INT0IF = 0;
	IEC0bits.INT0IE = 1;
	IPC0 = 1;


	//SPI2 Init
    SPI2STAT = 0;			
    SPI2CON1 = 0;
	SPI2CON1bits.SMP = 0;
	SPI2CON1bits.CKE = 1;				//1
	SPI2CON1bits.CKP = 0;				//0
    SPI2CON1bits.MODE16 = 0;
    SPI2CON1bits.MSTEN = 1;
    SPI2CON1bits.SPRE = 0b110;           //SPI divisor
    SPI2CON1bits.PPRE = 0b00;			//SPI Divisor
    SPI2STATbits.SPIEN = 1;
	RPOR4bits.RP43R = 0b001001;			//SPI CLOCK: RP43/RB11
	RPOR4bits.RP42R = 0b001000;			//SPI DATA OUT : RP42/RB10

	LCDInit();
	UARTInit();
	BTInit();
	initI2C();
	MPUInit();
	int dev = 0;
	dev = MPURead(MPU6050_ADDRESS, WHO_AM_I_MPU6050);

}

unsigned char MPUAvaibleData = 0;	//One when MPU interrupt 
short AccelData[3];					//Accelerometer data
short GyroData[3];					//Gyro data
float Degrees[] = {0,0,0};			//Calculated degrees


int main()
{
    init();	
	LCDClearText();
    while(1)
    {
		if(MPUAvaibleData == 1)
		{
			//MPU Handle
			MPUReadGyroData(MPU6050_ADDRESS, GyroData);
			MPUReadAccelData(MPU6050_ADDRESS, AccelData);
			
			//Calculate angle
			ComplementaryFilter(AccelData, GyroData, &Degrees[0], &Degrees[1]);

			//Bluetooth handle
			BTSend("Data:", 6);
			short Data = (short)Degrees[0];
			BTSend(&Data, sizeof(Data));
			Data = (short)Degrees[1];
			BTSend(&Data, sizeof(Data));
			BTSend("End", 4);

			//LCD handle
			LCDClearText();
			char number[7];
			char number2[7];
			memset(number,0,sizeof(number));
			memset(number,0,sizeof(number2));
			IntToChar(Degrees[0],number);
			IntToChar(Degrees[1], number2);
			LCDWriteAngle(number, number2);					
			MPUAvaibleData = 0;
		}
    }
    return 1;
}

char SPISend(char *Data)
{
	STATUS_LED = !STATUS_LED;
	SPI2BUF = *Data;
	while(SPI2STATbits.SPITBF);
	unsigned char Received = SPI2BUF;
	return Received;
}


void __attribute__((__interrupt__, no_auto_psv)) _INT0Interrupt(void)	//Read MPU avaible data
{
	MPUAvaibleData = 1;
	IFS0bits.INT0IF = 0;
}



